<?php

namespace Drupal\mimedetect;

use Drupal\Core\DependencyInjection\ServiceProviderBase;
use Drupal\file\FileInterface;

/**
 * MimeDetect service.
 */
class MimeDetectService extends ServiceProviderBase {

  /**
   * The PHP fileinfo instance.
   *
   * @var \finfo
   */
  protected $finfo;

  /**
   * Full UNIX 'file' command line.
   *
   * @var string
   */
  protected $unixfilecmd;

  /**
   * Construct the MimeDetectService object.
   */
  public function __construct() {
    $config = \Drupal::config('mimedetect.settings');

    // Enable fileinfo engine.
    if ($config->get('fileinfo.enable') && $this->testFileinfo($config->get('magicfile'))) {
      $this->finfo = new \finfo(FILEINFO_MIME_TYPE, $config->get('magicfile'));
    }

    // Enable UNIX 'file' command engine.
    if ($config->get('unixfile.enable') && $this->testUnixfile($config->get('unixfile.binary'), $config->get('magicfile'))) {
      $this->unixfilecmd = $config->get('unixfile.binary') . ' --brief --mime' . (!empty($config->get('magicfile')) ? ' --magic-file=' . escapeshellarg($config->get('magicfile')) : '') . ' ';
    }
  }

  /**
   * Get the MIME type for a given file.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file to be analyzed.
   *
   * @return string
   *   The detected MIME type for the given file.
   */
  public function getMime(FileInterface $file) {
    $mime = FALSE;
    $file_path = \Drupal::service('file_system')->realpath($file->getFileUri());

    // Try PHP fileinfo.
    if ($this->finfo) {
      $mime = $this->finfo->file($file_path);
    }

    // Try the 'file' UNIX command.
    if ((!$mime || $mime == 'application/octet-stream') && $this->unixfilecmd) {
      $mime = trim(exec($this->unixfilecmd . escapeshellarg($file_path)));
    }

    if ($mime) {
      // With text we often get charset like 'text/plain; charset=us-ascii'.
      $mime = explode(';', $mime);
      $mime = trim($mime[0]);
    }
    else {
      // Try file name extension mapping.
      $mime = \Drupal::service('file.mime_type.guesser')->guess($file_path);
    }

    return $mime ?: 'application/octet-stream';
  }

  /**
   * Test a 'magic' file information.
   *
   * @param string $magic_file
   *   Path to magic file.
   * @param string $msg
   *   Optional string variable reference to place error message.
   *
   * @return bool
   *   Test result. TRUE if OK, FALSE on error.
   */
  public static function testMagicfile($magic_file, &$msg = '') {
    $errors = FALSE;

    // Basic file exists check.
    if ($errors = !file_exists($magic_file)) {
      $msg = t("The path %path does not exist or is not readable by your webserver.", ['%path' => $magic_file]);
    }
    elseif ($errors = (filetype($magic_file) != 'file' || !@filesize($magic_file))) {
      $msg = t("Could not load the magic file %path.", ['%path' => $magic_file]);
    }

    return !$errors;
  }

  /**
   * Check availability for PHP fileinfo mime detection engine.
   *
   * @param string $magic_file
   *   An optional 'magic' information file.
   * @param string $msg
   *   Optional string variable reference to place error message.
   *
   * @return bool
   *   Test result. TRUE if OK, FALSE on error.
   */
  public static function testFileinfo($magic_file = '', &$msg = '') {
    // Test magic file if not empty.
    $errors = !empty($magic_file) ? !MimeDetectService::testMagicfile($magic_file, $msg) : FALSE;
    if ($errors) {
      return FALSE;
    }

    // Check extension availability.
    if ($errors = !extension_loaded('fileinfo')) {
      $msg = t('PHP file information extension not found.');
    }
    elseif ($errors = !@finfo_open(FILEINFO_MIME, $magic_file)) {
      if (!empty($magic_file)) {
        $msg = t('Fileinfo cannot load the configured magic file %path. It could be corrupted. Try another magic file or remove your magic file path settings to use defaults.', ['%path' => $magic_file]);
      }
      else {
        $msg = t('Fileinfo could not load magic information. Check the MAGIC environment variable on your system and that fileinfo PHP extension is properly installed.');
      }
    }

    return !$errors;
  }

  /**
   * Check UNIX 'file' command system avilability.
   *
   * @param string $bin_file
   *   Optional file path to the UNIX file command binary.
   * @param string $magic_file
   *   An optional 'magic' information file.
   * @param string $msg
   *   Optional string variable reference to place error message.
   *
   * @return bool
   *   Test result. TRUE if OK, FALSE on error.
   */
  public static function testUnixfile($bin_file = '/usr/bin/file', $magic_file = '', &$msg = '') {
    // Test magic file if not empty.
    $errors = !empty($magic_file) ? !MimeDetectService::testMagicfile($magic_file, $msg) : FALSE;
    if ($errors) {
      return FALSE;
    }

    if ($errors = empty($bin_file)) {
      $msg = t("You must specify the path to the 'file' binary if it is enabled.");
    }
    elseif ($errors = basename($bin_file) != 'file') {
      $msg = t("Base name %basename for the 'file' binary not allowed. Must be 'file'.", ['%basename' => basename($bin_file)]);
    }
    elseif ($errors = !file_exists($bin_file)) {
      $msg = t("The path %path does not exist or is not readable by your webserver.", ['%path' => $bin_file]);
    }
    elseif ($errors = !is_executable($bin_file)) {
      $msg = t("%path is not executable by your webserver.", ['%path' => $bin_file]);
    }
    else {
      // Execution test.
      $exit_code = 0;
      $empty = NULL;
      exec($bin_file . (!empty($magic_file) ? ' --magic-file=' . escapeshellarg($magic_file) : '') . ' --list', $empty, $exit_code);
      if ($errors = $exit_code !== 0) {
        if (!empty($magic_file)) {
          $msg = t('File command execution failed with exit code %code. Could not load the magic file %file.', ['%code' => $exit_code, '%file' => $magic_file]);
        }
        else {
          $msg = t('File command execution failed with exit code %code.', ['%code' => $exit_code]);
        }
      }
    }

    return !$errors;
  }

}
